Tree

Struct Tree 

pub struct Tree<I: Instance, U: Number, D: Dataset<I, U>, C: Cluster<U>> { /* private fields */ }
A Tree represents a hierarchy of Clusters, i.e. “similar” instances from a metric-Space.

The Tree has other implementation blocks spread across the crate. These are used for specific functionality for the concrete Cluster types we provide.

Type Parameters

  • T: The type of the instances in the Tree.
  • U: The type of the distance values between instances.
  • D: The type of the Dataset from which the Tree is built.

Implementations

Source

impl<I: Instance, U: Number, D: Dataset<I, U>> Tree<I, U, D, Vertex<U>>

pub fn normalize_ratios(self) -> Self

Sets the Vertex ratios for anomaly detection and related applications.

This should only be called on the root Cluster after calling partition.

Arguments
  • normalized: Whether to apply Gaussian error normalization to the ratios.
Source

impl<I: Instance, U: Number, D: Dataset<I, U>, C: Cluster<U>> Tree<I, U, D, C>

pub fn new(data: D, seed: Option<u64>) -> Self

Constructs a new Tree for a given dataset. Importantly, this does not partition the tree.

Arguments

dataset: The dataset from which the tree will be built

pub fn partition<P: PartitionCriterion<U>>( self, criteria: &P, seed: Option<u64>, ) -> Self

Recursively partitions the root Cluster using the given criteria.

Arguments
  • criteria: the criteria used to decide when to partition a Cluster.
Returns

The Tree after partitioning.

pub fn get_cluster(&self, offset: usize, cardinality: usize) -> Option<&C>

Returns the Cluster with the given offset and cardinality.

Arguments
  • offset: The offset of the Cluster to return.
  • cardinality: The cardinality of the Cluster to return.
Returns

The Cluster with the given offset and cardinality if it exists. Otherwise, None.

pub const fn data(&self) -> &D

Returns a reference to the data used to build the Tree.

pub fn cardinality(&self) -> usize

The cardinality of the Tree, i.e. the number of instances in the data.

pub fn radius(&self) -> U

The radius of the root of the Tree.

pub const fn root(&self) -> &C

The root Cluster of the Tree.

pub const fn depth(&self) -> usize

The depth of the Tree.

pub fn save(&self, path: &Path) -> Result<(), String>

Saves a tree to a given location

The path given will point to a newly created folder which will store all necessary data for tree reconstruction.

The directory structure looks like the following:

/user/given/path/
   |- dataset      <-- The serialized dataset.
   |- clusters     <-- Clusters are serialized to a single file.
Arguments
  • path - The path to save the tree to.
Errors
  • If path does not exist.
  • If path cannot be written to.
  • If there are any serialization errors with the dataset.
pub fn load( path: &Path, metric: fn(&I, &I) -> U, is_expensive: bool, ) -> Result<Self, String>

Reconstructs a Tree from a directory path with associated metric metric. Returns the reconstructed tree.

Arguments
  • path - The path to load the tree from.
  • metric - The metric to use for the tree.
  • is_expensive - Whether or not the metric is expensive to compute.
Returns

The reconstructed tree.

Errors
  • If path does not exist.
  • If path does not contain a valid tree. See save for more information on the directory structure.
  • If the path cannot be read from.
  • If there are any deserialization errors with the dataset.
  • If there are any deserialization errors with the clusters.